/**
    \file 
        TestCullingApplication.h
*/
#include "CEGUIForwardRefs.h"
#include "ExampleApplication.h"
#include "VisibilityEnvironment.h"
#include "SceneContentGenerator.h"

Real timeDelay = 0;
#define KEY_PRESSED(_key,_timeDelay, _macro) \
{ \
    if (mInputDevice->isKeyDown(_key) && timeDelay <= 0) \
{ \
    timeDelay = _timeDelay; \
    _macro ; \
} \
}

String mCurrentAlgorithmCaptions[GtpVisibility::VisibilityEnvironment::NUM_CULLING_MANAGERS] = 
{
    "View Frustum Culling", 
	"Stop and Wait Culling", 
	"Coherent Hierarchical Culling"
};

class MouseQueryListener : public ExampleFrameListener, public MouseListener, public MouseMotionListener
{
public:

    MouseQueryListener(RenderWindow* win, Camera* cam, SceneManager *sceneManager, 
			CEGUI::Renderer *renderer, SceneContentGenerator *sceneContentGenerator);


    ~MouseQueryListener( );

	bool frameStarted(const FrameEvent& evt);
	bool frameEnded(const FrameEvent& evt);

    /* MouseListener callbacks. */
    virtual void mouseClicked(MouseEvent* e) { }
    virtual void mouseEntered(MouseEvent* e) { }
    virtual void mouseExited(MouseEvent* e)  { }

    // This is when the mouse button goes DOWN.
    virtual void mousePressed(MouseEvent* e);

    // This is when the mouse button is let UP.
    virtual void mouseReleased(MouseEvent* e);

    /* MouseMotionListener callbacks */
    virtual void mouseMoved (MouseEvent *e);
   
    // This is when the mouse is clicked, held and dragged.
    virtual void mouseDragged (MouseEvent *e);

    void keyPressed(KeyEvent* e);

	void keyReleased(KeyEvent* e);
	void keyClicked(KeyEvent* e);

	void nextAlgorithm();
	void setAlgorithm(int algorithm);
	void changeThreshold(int incr);
	void changeStats();

protected:
    bool mLMouseDown, mRMouseDown;     // True if the mouse buttons are down
    SceneManager *mSceneMgr;           // A pointer to the scene manager
    
	CEGUI::Renderer *mGUIRenderer;     // cegui renderer
	
	bool mShutdownRequested;
	int mCurrentAlgorithm;
	int mVisibilityThreshold;

	OverlayElement *mAlgorithmInfo;
	OverlayElement *mThresholdInfo;
	OverlayElement *mFrustumCulledNodesInfo;
	OverlayElement *mQueryCulledNodesInfo;
    OverlayElement *mTraversedNodesInfo;
	OverlayElement *mHierarchyNodesInfo;
	OverlayElement *mRenderedNodesInfo;
	OverlayElement *mNumObjectsInfo;

	SceneContentGenerator *mSceneContentGenerator;
};


class TestCullingApplication : public ExampleApplication
{
public:
	~TestCullingApplication();

protected:
	void createScene();
	void createFrameListener();
	void setupGui();
	
	//virtual void createCamera(void);

	CEGUI::OgreCEGUIRenderer *mGUIRenderer;
	CEGUI::System *mGUISystem;

	Vector3 mMinTranslation;
	Vector3 mMaxTranslation;

	Vector3 mMinAngle;
	Vector3 mMaxAngle;

	SceneContentGenerator *mSceneContentGenerator;

private:
	void chooseSceneManager(void);
};

